/***********************************************************************/
/*                                                                     */
/*                               Pio (C)                               */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2011-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Bruno Verlyck <Bruno.Verlyck@inria.fr>                             */
/*                                                                     */
/***********************************************************************/

#ifndef PIO_IBUFF_INCLUDED
#define PIO_IBUFF_INCLUDED

#include "pio_buffer.h"

#include <stdarg.h>
#include <stdio.h>

typedef struct {
  bool IB_at_eof;
  int IB_nc_scanned;	/* temp place to store (tentative) sscanf result */
  int __fill;		/* bug vsscanf ? */
  FILE *IB_stream;	/* NULL if source was a string (we can't refill) */
  size_t IB_endpos;	/* useful data ends here (== buf_length if IB_from_string) */
  pio_buffer IB_buf;
} pio_input_buffer;

void ibuf_of_stream (pio_input_buffer *ibuf, FILE *stream);
void ibuf_of_string (pio_input_buffer *ibuf, struct string *str);

bool refill (pio_input_buffer *ib); /* for r_float */

jmp_buf *eof_bscanf_label;
/* handler to externally decide if scanning is ok after all */
bool (*on_unknown_scan) (pio_input_buffer *);

/* format must be a litteral; if not, use __ibuf_scanf */
#define ibuf_scanf(ib, ntok, format, ...) __ibuf_scanf (ib, ntok, format "%n", __VA_ARGS__, &ib->IB_nc_scanned)
/* if we expect no token */
#define ibuf_scanf0(ib, format) __ibuf_scanf (ib, 0, format "%n", &ib->IB_nc_scanned)
void __ibuf_scanf (pio_input_buffer *ib, const size_t ntok_expected, const char *format, ...);

struct string read_delimited_string (pio_input_buffer *ib, size_t initial);

#endif
