(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2005-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of ocamlprintc.bin generated code *)

let print_values ppf = function
  | t_t_a -> Lib_print.print_Array Typed_value_print.print_t ppf t_t_a
;;

let print_service_name ppf = function
  | Communication.Ok -> Format.fprintf ppf "%s" "Ok"
  | Communication.Ko -> Format.fprintf ppf "%s" "Ko"
  | Communication.Allo -> Format.fprintf ppf "%s" "Allo"
  | Communication.Bye -> Format.fprintf ppf "%s" "Bye"
  | Communication.Start -> Format.fprintf ppf "%s" "Start"
  | Communication.Stop -> Format.fprintf ppf "%s" "Stop"
  | Communication.WrongCommunication ->
    Format.fprintf ppf "%s" "WrongCommunication"
;;

let print_task_name ppf = function | s -> Lib_print.print_quoted_string ppf s
;;

let print_t ppf = function
  | Communication.Phrase v ->
    Format.fprintf ppf "@[<1>%s@ " "Phrase"; print_values ppf v;
    Format.fprintf ppf "%,@]"
  | Communication.Task (t, v) ->
    Format.fprintf ppf "@[<1>%s@ " "Task"; Format.fprintf ppf "%,@[<1>(@,";
    print_task_name ppf t; Format.fprintf ppf "%,,@ "; print_values ppf v;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Communication.Result v ->
    Format.fprintf ppf "@[<1>%s@ " "Result"; print_values ppf v;
    Format.fprintf ppf "%,@]"
  | Communication.Error v ->
    Format.fprintf ppf "@[<1>%s@ " "Error"; print_values ppf v;
    Format.fprintf ppf "%,@]"
  | Communication.Service (s, v) ->
    Format.fprintf ppf "@[<1>%s@ " "Service";
    Format.fprintf ppf "%,@[<1>(@,"; print_service_name ppf s;
    Format.fprintf ppf "%,,@ "; print_values ppf v;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
;;

(* End of ocamlprintc.bin generated code *)
