(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2005-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options_pio.ml,v 1.12 2014/01/10 19:22:10 fclement Exp $ *)

(* Report/debug flags and error messages. *)

let purpose = Udebug.Library;;
let software_name = "Pio/OCaml";;
let key_prefixes = Udebug.make_key_prefixes software_name;;

let {
  Udebug.report = report;
  report_action = report_action;
  debug = debug;
  debug_action = debug_action;
  fatal_error = fatal_error;
} = Udebug.make_printing_facilities purpose software_name key_prefixes
;;

(* Binary/ASCII modes. *)

let ({ Uaref.get_tflag = get_binary_mode; toggle_tflag = set_binary_mode; _ },
     { Uaref.get_tflag = get_ascii_mode; toggle_tflag = set_ascii_mode; _ }) =
  Uaref.make_antonymic_toggle_flags true;;

let my_regular_opt rco =
  Ucommand_option.with_auto_short_key (Ucommand_option.of_regular rco)
;;

let my_library_add_all l =
  Ucommand_option.library_add_all (List.map my_regular_opt l)
;;

my_library_add_all
  [
   {
    Ucommand_option.key =
      Printf.sprintf "-%s-ascii" key_prefixes.Udebug.long;
    spec = Arg.Unit set_ascii_mode;
    doc =
      Printf.sprintf
        "  Turn on the %s ascii mode:\
        \n\tfloating-point numbers are written using an ascii \
        human-readable format.\
        \n\tThis mode is exclusive to binary mode. Default is %b."
        software_name (get_ascii_mode ());
   };
   
   {
    Ucommand_option.key =
      Printf.sprintf "-%s-binary" key_prefixes.Udebug.long;
    spec = Arg.Unit set_binary_mode;
    doc =
      Printf.sprintf
        "  Turn on the %s binary mode:\
        \n\tfloating-point numbers are written using a compact binary \
        format.\
        \n\tThis mode is exclusive to ascii mode. Default is %b."
        software_name (get_binary_mode ());
   };
  ]
;;
