(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.11 2014/01/21 09:21:18 fclement Exp $ *)

open Rdec2;;
open Config;;
open Gnuplot;;

let { Uaref.get = get_number_of_workers; set = set_number_of_workers; } =
  Uaref.make_aref_gen
    (Uaref.with_int_bounds (Some 1) None
      (Uaref.with_name "number_of_workers"
        (Uaref.with_single_assignment
          (Uaref.aref_spec 2))))

and { Uaref.get = get_config_dir; set = set_config_dir; } =
  Uaref.make_aref_single "config_dir" (Filename.concat "." "data")

and { Uaref.get = get_config_filename; set = set_config_filename; } =
  Uaref.make_aref_single "config_filename" "config"
;;

let get_global_parameters () =
  let cfg_fname =
    Filename.concat (get_config_dir ()) (get_config_filename ()) in
  let cfg = parse_configuration_file cfg_fname in
  let rdec_fname =
    Filename.concat (get_config_dir ()) cfg.rdec_config_filename in
  let rdec = parse_rdec_config_file rdec_fname in
  check_regular_decomposition rdec;
  check_graphical_output cfg.graphical_output;
  let nsbdx, nsbdy = number_of_subdomains_of_regular_decomposition rdec in
  set_number_of_workers (nsbdx * nsbdy);
  cfg_fname, cfg, rdec
;;

let add_options () =
  Ucommand_option.user_add_all
    [
     Ucommand_option.with_auto_short_key
       (Ucommand_option.of_regular
          {
           Ucommand_option.key = "--configuration-directory";
           spec = Arg.String set_config_dir;
           doc =
             Printf.sprintf
               "<dir>  Set the configuration directory to <dir>.\
               \n\tDefault directory is %S."
               (get_config_dir ());
          });
     
     Ucommand_option.with_auto_short_key
       (Ucommand_option.of_regular
          {
           Ucommand_option.key = "--configuration-filename";
           spec = Arg.String set_config_filename;
           doc =
             Printf.sprintf
               "<fname>  Set the configuration filename to <fname>.\
               \n\tDefault file is %S."
               (get_config_filename ());
          });
    ];
;;

Ucommand_option.set_usage_msg
  (Printf.sprintf
     "\n%s -- Domain decomposition, solves Poisson's equation"
     Sys.argv.(0))
;;
