(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: hello.ml,v 1.14 2014/01/21 09:21:18 fclement Exp $ *)

open Skl;;

let stream_of_string s =
  let rec explode i =
    if i = String.length s then [] else
    s.[i] :: explode (succ i)
  in
  Sklstream.of_list (explode 0)
;;

let id x = x;;

let put_char c = Printf.printf "%c%!" c; c;;

(* The atomic put char skeleton. *)
let put_char_skl = skl b -> if b then put_char else id;;

(* The full computational skeleton expression. *)
let hello_skl b nw = farm (put_char_skl b, nw);;

let main { Parfun.parfun = parfun; } =
  let input_stream = stream_of_string "Hello world!\n" in
  let output_stream = parfun (hello_skl true 10) input_stream in
  Sklstream.iter ignore output_stream;
  let output_stream = parfun (hello_skl false 10) input_stream in
  Sklstream.iter (fun x -> ignore (put_char x)) output_stream;
;;

Parfun.pardo main;;
