(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.24 2014/01/21 09:21:18 fclement Exp $ *)

open Skl;;

(* Pi pretty printer *)
let pppi r =
  let num = Ratio.numerator_ratio r
  and den = Ratio.denominator_ratio r in
  let mul =
    Big_int.power_int_positive_int
      10 (Options.get_number_of_decimals ()) in
  let decimals =
    let num = Big_int.mult_big_int mul num in
    Big_int.(string_of_big_int (div_big_int num den)) in
  let output = Printf.sprintf "%c.%s"
    decimals.[0] (String.sub decimals 1 (String.length decimals - 1)) in
  let rec format s =
    let n = String.length s in
    if n <= Options.chars_per_col then s else
    String.sub s 0 Options.chars_per_col ^ "\n" ^
      format (String.sub s Options.chars_per_col (n - Options.chars_per_col))
  in
  format output
;;

let rec list_init f n =
  if n = 0 then [] else
  let n = n - 1 in
  f n :: list_init f n
;;

let start_vector it nw =
  let rec aux i =
    if i = 0 then [] else
    let shift = if i <= it mod nw then 1 else 0 in
    let n = it / nw + shift in
    let l = List.map (fun x -> x * nw + i - 1)
      (list_init (fun x -> x) n) in
    l :: aux (i - 1)
  in
  Array.of_list (aux nw)
;;

(* The atomic run steps skeleton. *)
let run_steps_skl = skl () -> Bbp.run_steps;;

(* The full computational skeleton expression. *)
let pigen_skl nw = Skl_extra.rails_same (run_steps_skl (), nw);;

let main { Parfun.parfun = parfun; } =
  Ucommand_option.parse ();
  let it = Options.get_number_of_iterations () in
  let nw = min (Options.get_number_of_workers ()) (it / 2) in
  let input_stream = Sklstream.singleton (start_vector it nw) in
  let output_stream = parfun (pigen_skl nw) input_stream in
  print_string "Computed decimals :\n";
  Sklstream.iter
    (fun sa ->
      let a = Array.map Ratio.ratio_of_string sa in
      let digits =
        pppi (Array.fold_left Ratio.add_ratio (Ratio.ratio_of_int 0) a) in
      Printf.printf "%s\n" digits)
    output_stream;
;;

Options.add_options ();;
Parfun.pardo main;;
