(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2012 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: pool.mli,v 1.6 2012/02/17 14:38:55 fclement Exp $ *)

type 'a t
(** A pool is a persistant data structure which maps an integer to a
  value of type ['a]. A pool can be safely be shared by different
  processes. *)

val create : string -> string -> int -> 'a t
(** [Pool.create dir tdir n] creates a pool of data using [dir] as an
  exchange directory and [tdir] as a temporary directory. [n] plays 
  the same role as in {!Hashtbl.create}.
  *)

val find : 'a t -> int -> 'a
(** [Pool.find pool i] will lookup the value pointed by [i] in the
  pool [pool]. If no value is found, [Not_found] is raised. *)

val add : 'a t -> int -> 'a -> unit
(** [Pool.add pool i x] will add the value [x] pointed by [i] in the
  pool [pool].*)

val clear_cache : 'a t -> unit
(** [Pool.clear_cache pool] will clear the internal cache of the pool
  [pool]. Depending on how you use the pool, the cache might be in a
  corrupted state, in this case use [clear_cache]. *)
