(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl_extra.ml,v 1.8 2014/01/21 09:21:16 fclement Exp $ *)

(* If you can build an interesting skeleton from existing ones, put it
   here *)

open Skl;;

let from_Some o = match o with Some x -> x | None -> invalid_arg "from_Some";;

let rails_same (skl, n) = rails (Array.make n skl);;

let dup : 'a. unit -> ('a, ('a * 'a)) Skl.t =
  let dup_pure = fun () ->
    Sklstream.map (fun x -> (x, x)) in
  ignore (Expor.register dup_pure);
  fun () -> atom ((), from_Some (Expor.is_registered dup_pure))
;;

let projl : unit -> (('a * 'b), 'a) Skl.t =
  let projl_pure = fun () -> Sklstream.map fst in
  ignore (Expor.register projl_pure);
  fun () -> atom ((), from_Some (Expor.is_registered projl_pure))
;;

let projr : unit -> (('a * 'b), 'b) Skl.t =
  let projr_pure = fun () -> Sklstream.map snd in
  ignore (Expor.register projr_pure);
  fun () -> atom ((), from_Some (Expor.is_registered projr_pure))
;;

let injl : unit -> ('a, ('a, 'b) sum) Skl.t =
  let injl_pure = fun () -> Sklstream.map (fun x -> Inl x) in
  ignore (Expor.register injl_pure);
  fun () -> atom ((), from_Some (Expor.is_registered injl_pure))
;;

let injr : unit -> ('b, ('a, 'b) sum) Skl.t =
  let injr_pure = fun () -> Sklstream.map (fun x -> Inr x) in
  ignore (Expor.register injr_pure);
  fun () -> atom ((), from_Some (Expor.is_registered injr_pure))
;;

(* Skeletons used by if_then_else *)
let to_sum : unit -> (('a * bool), ('a, 'a) sum) Skl.t =
  let to_sum_pure = fun () ->
    Sklstream.map (fun (x, b) -> if b then Inl x else Inr x) in
  ignore (Expor.register to_sum_pure);
  fun () -> atom ((), from_Some (Expor.is_registered to_sum_pure))
;;


let if_then_else (cond_skl, then_skl, else_skl) =
  dup () ||| (id () *** cond_skl) ||| to_sum () ||| (then_skl +++ else_skl)
;;
